/************************************************************************
 *									*
 * (C) Copyright 2001 - Analog Devices, Inc.  All rights reserved.	*
 *									*
 * File Name:    SPI_Master.asm						*
 *									*
 * Date  : 6/22/01							*
 * Author: J.Beauchemin							*
 * Rev # : 0.0								*
 *									*
 * PURPOSE: SPI Master routine for the ADSP-2191			*
 *									*
 * HARDWARE: Use 2 ADSP-2191 DSPs, wire master PF4 to slave PF0		*
 *	  Wire master SPI0_MOSI to slave SPI0_MOSI 			*
 *	  Wire master SPI0_MISO to slave SPI0_MISO 			*
 *	  Wire master SPI0_SCK  to slave SPI0_SCK  			*
 *									*
 ************************************************************************/
 
/* Line 20 */ #include <def2191.h>

/************************************************************************
 * GLOBAL & EXTERNAL DECLARATIONS					*
 ************************************************************************/ 
.GLOBAL Start;

/************************************************************************
 * 				DM data 				*
 ************************************************************************/
/* Line 30 */ .section/data data1;			/* Place Your Transmit Data Here*/
.var TX_Buf_MASTER[16] = 0x0000, 0x1111, 0x2222, 0x3333, 0x4444, 0x5555, 0x6666, 0x7777, 0x8888, 0x9999, 0xAAAA, 0xBBBB, 0xCCCC, 0xDDDD, 0xEEEE, 0xFFFF;
.var RX_Buf_MASTER[16];                 /* Buffer For SPI0 Receive Data */

/************************************************************************  
 * 			Program memory code	 			*
 ************************************************************************/  
.SECTION /pm program;
Start:
_main:
/* Line 40 */	call Program_SPI0_Interrupt; /* Initialize Interrupt Priorities */	
	call SPI0_Register_Initialization;	     /* Initialize SPI0 */
	call Initiate_Transfers;		      /* Start Transfer */

wait_forever:
	jump wait_forever;

/************************************************************************
 * INTERRUPT PRIORITY CONFIGURATION					*
 ************************************************************************/ 
/* Line 50 */ .section/code program;		// PROGRAM STARTS HERE
Program_SPI0_Interrupt:
  	IOPG = 0;
  	ar=io(SYSCR);
  	ar = setbit 4 of ar;	 /* Map Interrupt Vector Table to Page 0*/
	ar = setbit 0 of ar;	      /* Turn on SPI Ports (Not SPORT2) */
  	io(SYSCR)=ar;

	DIS int;			      /* Disable all interrupts */
  	IRPTL = 0x0;				/* Clear all interrupts */
/* Line 60 */  	ICNTL = 0x0;			   /* Interrupt nesting disable */
  	IMASK = 0;				 /* Mask all interrupts */
	
	/* Setup Interrupt Priorities */
   	IOPG = Interrupt_Controller_Page;	
	ar = 0xBB1B;			   /* Assign SPI0 priority of 1 */
  	io(IPR1) = ar;
  	ar = 0xBBBB;	   /* Assign the remainder with lowest priority */
  	io(IPR0) = ar;
  	io(IPR2) = ar;						
/* Line 70 */	io(IPR3) = ar; 	

	AY0=IMASK;
	AY1=0x0020;				/* Unmask SPI Interrupt */
	AR = AY0 or AY1;						
	IMASK=AR;

	RTS;
	
/************************************************************************
 * SPI0 REGISTER INTIALIZATION	- MASTER				*
 ************************************************************************/
.SECTION /pm program;
SPI0_Register_Initialization:

/* Line 85 */	IOPG = SPI0_Controller_Page; /* Set I/O Page To Change Registers*/
	
	/* Set Up Device Select Over SPI0 Interface */
	AR = 0xFF04;	     /* Enable Slave On Programmable Flag Pin 4 */
	IO(SPIFLG0) = AR;	 	 /* Write to SPI0 Flag Register */

	/* Write to SPI0 Baud rate register */
	AR = 0x1047;		
	IO(SPIBAUD0) = AR;			       /* SCLK0 ~= 9600 */

	/* Setup SPI0 Configuration Reigster */
/************************************************************************
 * TIMOD [1:0] = 00 : Transfer On RDBR Read. Interrupt when RDBR Full	*
 * SZ [2]      =  0 : Send Last Word When TDBR0 Is Empty		*
 * GM [3]      =  1 : Discard Incoming Data If RDBR0 Is Full		*
 * PSSE [4]    =  0 : Disables Slave-Select As Input (Master)		*
 * EMISO [5]   =  0 : MISO Disabled For Output (Master)			*
 * [7] and [6] =  0 : RESERVED						*
 * SIZE [8]    =  1 : 16 Bit Word Length Select				*
 * LSBF [9]    =  0 : Transmit MSB First				*
 * CPHA [10]   =  1 : SCLK Starts Toggling At START Of First Data Bit	*
 * CPOL [11]   =  1 : Active HIGH Serial Clock				*
 * MSTR [12]   =  1 : Device Is Master					*
 * WOM [13]    =  0 : Normal MOSI/MISO Data Output (No Open Drain)	*
 * SPE [14]    =  1 : SPI Module Is Enabled				*
 * [15]        =  0 : RESERVED						*
 ************************************************************************/

/* Line 113 */	AR = 0x5D08;
	IO(SPICTL0) = AR;		       /* Enable SPI0 as MASTER */
	RTS;
	
/************************************************************************
 * INITIATE AND START TRANSFER						*
 ************************************************************************/
.SECTION /pm program;
Initiate_Transfers:
	/* Initialize DAG registers */
/* Line 123 */	I0 = TX_Buf_MASTER;		   /* Pointer to TX SPI0 Buffer */
	I1 = RX_Buf_MASTER;		   /* Pointer to RX SPI0 Buffer */
	M1 = 1;
	L0 = 0;				 	   /* Linear Addressing */
	L1 = 0;	
	
	ENA INT;	 		  /* Globally enable interrupts */
	IOPG = SPI0_Controller_Page;
	AR = 0xFB04;			   /* Drive Slave Select On PF4 */
	IO(SPIFLG0) = AR;		      /* Write To Flag Register */
/* Line 133 */	AR = IO(RDBR0);	      /* Dummy read from SPI0 to kick off x-fer */
    
	CNTR = length(TX_Buf_MASTER);  /* loop to xmit entire TX buffer */

	DO looping until ce;
	AR = DM(I0, M1);	     /* Get Next Data To Be Transmitted */
	IO(TDBR0) = AR;		       /* Write to SPI0 Transmit Buffer */
	
looping:	idle;		     /* Wait for SPI0 Receive Interrupt */
	
/* Line 143 */	DIS INT;			 /* Globally disable Interrupts */
	AR = 0xFF00;		      /* Disable and Clear Slave Select */
	io(SPIFLG0) = AR;
	RTS;